@@ -1,7 +1,7 @@
 /*
  * console.c
  *
- * Copyright 2000,2015 Werner Fink, 2000 SuSE GmbH Nuernberg, Germany.
+ * Copyright 2000,2015,2025 Werner Fink, 2000 SuSE GmbH Nuernberg, Germany.
  * Copyright 2015 SuSE Linux GmbH.
  *
  * This source is free software; you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 #include <string.h>
 #include <sys/epoll.h>
 #include <sys/ioctl.h>
+#include <sys/klog.h>
 #include <sys/mman.h>
 #include <sys/prctl.h>
 #include <sys/socket.h>
@@ -49,24 +50,35 @@
 # define _PATH_BLOG_FIFO	"/dev/blog"
 #endif
 
-#if defined(__s390__)
-# define RED	""
-# define BOLD	">> "
-# define NORM	""
-#else
-# define RED	"\e[31m"
-# define BOLD	"\e[1m"
-# define NORM	"\e[m"
-#endif
+#define RED	"\e[31m"
+#define BOLD	"\e[1m"
+#define NORM	"\e[m"
 
+/* Disable printk's to console */
+#define SYSLOG_ACTION_CONSOLE_OFF	6
+/* Enable printk's to console */
+#define SYSLOG_ACTION_CONSOLE_ON	7
+/* Set level of messages printed to console */
+#define SYSLOG_ACTION_CONSOLE_LEVEL	8
+/* Return number of unread characters in the log buffer */
+#define SYSLOG_ACTION_SIZE_UNREAD	9
+
+/*
+ * Password/Passphrase is asked if true.
+ */
+volatile sig_atomic_t asking;
+
+/*
+ * Move log file to old file
+ */
 int final = 0;
-static volatile char *_arg0;
 
 /*
  * Avoid trouble if linked with e.g. blogger as there
  * is no external arg0 but linker on ppc64 and s390/x
  * seems to expect this.
  */
+static volatile char *_arg0;
 void remember_arg0(volatile char *arg0)
 {
     _arg0 = arg0;
@@ -123,10 +135,10 @@ void safeout (int fd, const void *ptr, size_t s, ssize_t max)
 	ssize_t p;
 	if (issocket) {
 	    int flags = MSG_NOSIGNAL;
-            if (s > max)
+	    if (s > max)
 		flags |= MSG_MORE;
 	    p = send (fd, ptr, (max < 1) ? 1 : ((s < (size_t)max) ? s : (size_t)max), flags);
-        } else
+	} else
 	    p = write (fd, ptr, (max < 1) ? 1 : ((s < (size_t)max) ? s : (size_t)max));
 	if (p < 0) {
 	    if (errno == EPIPE)
@@ -257,10 +269,19 @@ static void sigio(int sig)
 }
 
 /*
- * Our transfer buffer
+ * Our transfer buffer from system console to the devices
  */
 static char trans[TRANS_BUFFER_SIZE];
 
+/*
+ * Our temporary buffer during asking a password/passphrase
+ */
+static	     char temp[4*TRANS_BUFFER_SIZE];
+static const char *const tend = temp + sizeof(temp);
+static       char *     thead = temp;
+static       char *     ttail = temp;
+static volatile ssize_t tavail;
+
 /*
  * Prepare I/O
  */
@@ -389,7 +410,10 @@ void safeIO (void)
 
 	    ret = lstat("/var/log", &st);
 	    if (ret < 0) {
-		warn("can not get file status of /var/log");
+		if (errno != ENOENT)
+		    warn("can not get file status of /var/log: %m");
+		else
+		    atboot = 1;
 		goto skip;
 	    }
 	    if ((st.st_mode & S_IFMT) == S_IFLNK) {
@@ -562,13 +586,13 @@ static int consinitIO(struct console *newc)
 {
     int tflags;
 
-    if ((newc->fd = open(newc->tty, O_WRONLY|O_NONBLOCK|O_NOCTTY)) < 0) {
+    newc->fd = open_tty(newc->tty, O_WRONLY|O_NONBLOCK|O_NOCTTY);
+    if (newc->fd < 0) {
 	if (errno == EACCES)
 	    error("can not open %s", newc->tty);
 	warn("can not open %s", newc->tty);
 	return 0;
     }
-
     newc->tlock = 0;
     newc->max_canon = _POSIX_MAX_CANON;
     memset(&newc->ltio, 0, sizeof(newc->ltio));
@@ -636,8 +660,13 @@ void getconsoles(struct console **cons, int io)
     char fbuf[16], dev[64];
     char *tty = NULL;
     FILE *fc;
+#ifdef TIOCGDEV
+    unsigned int devnum;
+    int fd;
+#endif
     int items;
 
+
     if (!cons)
 	error("error: console pointer empty");
 
@@ -652,8 +681,7 @@ void getconsoles(struct console **cons, int io)
     while ((items = fscanf(fc, "%*s %*s (%[^)]) %[0-9:]", &fbuf[0], &dev[0]))
 	   != EOF) {
 	char *tmp;
-	int flags, n, maj, min;
-	int ret;
+	int flags, n, maj, min, ret;
 
 	/* Ignore consoles without tty binding. */
 	if (items != 2)
@@ -667,24 +695,23 @@ void getconsoles(struct console **cons, int io)
 	    if (strchr(fbuf, con_flags[n].name))
 		flags |= con_flags[n].flag;
 
-	ret = asprintf(&tty, "/dev/char/%s", dev);
+	ret = asprintf(&tmp, "/dev/char/%s", dev);
 	if (ret < 0)
 	    error("can not allocate string");
 
-	tmp = tty;
-	tty = realpath(tty, NULL);
+	tty = realpath(tmp, NULL);
 	if (!tty) {
 	    if (errno != ENOENT && errno != ENOTDIR)
-		error("can not determine real path of %s", tmp);
+		error("can not determine real path of %s: %m", tmp);
 
 	    tty = charname(dev);
 	    if (!tty)
-		error("can not determine real path of %s", tmp);
+		error("can not determine real path of %s: %m", tmp);
 	}
 	free(tmp);
 
 	if (sscanf(dev, "%u:%u", &maj, &min) != 2)
-	    error("can not determine device numbers for %s", tty);
+	    error("can not determine device numbers for %s: %m", tty);
 
 	consalloc(&c, tty, flags, makedev(maj, min), io);
 	free(tty);
@@ -695,15 +722,35 @@ void getconsoles(struct console **cons, int io)
     if (!c)
 	goto err;
 
-     *cons = c;
+    *cons = c;
     return;
 err:
+#ifdef TIOCGDEV
+    fd = open("/dev/console", O_RDONLY|O_NONBLOCK|O_NOCTTY|O_CLOEXEC);
+    if (fd >= 0) {
+	if (ioctl (0, TIOCGDEV, &devnum) < 0)
+	    goto fallback;
+	close(fd);
+	tty = chardev((dev_t)devnum);
+	if (!tty)
+	    goto fallback;
+
+	if (!consalloc(&c, tty, CON_CONSDEV, (dev_t)devnum, io))
+	    error("/dev/console is not a valid fallback\n");
+	free(tty);
+
+	*cons = c;
+	return;
+    }
+fallback:
+#endif
     tty = strdup("/dev/console");
     if (!tty)
 	error("can not allocate string");
 
     if (!consalloc(&c, tty, CON_CONSDEV, makedev(TTYAUX_MAJOR, 1), io))
 	error("/dev/console is not a valid fallback\n");
+    free(tty);
 
     *cons = c;
 }
@@ -741,13 +788,52 @@ static void epoll_console_in(int fd)
 
 	parselog(trans, cnt);				/* Parse and make copy of the input */
 
+	/*
+	 * During asking a password/passphrase we are temporary
+	 * buffer console output to release it if we've got an answer.
+	 */
+	if (asking) {
+	    if (cnt <= (size_t)(tend - ttail)) {
+		memcpy(ttail, trans, cnt);
+		tavail = (ttail += cnt) - thead;
+	    }
+
+	    goto flush;                                 /* Temporary silent as waiting on passphrase */
+
+	} else while (tavail > 0) {			/* Empty temporary buffer if any */
+	    size_t len = (size_t)tavail;
+
+	    if (tavail > TRANS_BUFFER_SIZE)
+		len = TRANS_BUFFER_SIZE;
+
+	    list_for_each_entry(c, &cons->node, node) {
+		if (c->fd < 0)
+		    continue;
+		safeout(c->fd, thead, len, c->max_canon);
+		(void)tcdrain(c->fd);			/* Write copy of input to real tty */
+	    }
+	    thead += len;
+
+	    if (thead >= ttail) {
+		ttail = thead = temp;
+		tavail = 0;
+		break;
+	    }
+
+	    if (thead > temp) {				/* Buffer not empty, move contents */
+		tavail = ttail - thead;
+		thead = (char *)memmove(temp, thead, tavail);
+		ttail = thead + tavail;
+	    }
+	}
+
 	list_for_each_entry(c, &cons->node, node) {
 	    if (c->fd < 0)
 		continue;
 	    safeout(c->fd, trans, cnt, c->max_canon);
 	    (void)tcdrain(c->fd);			/* Write copy of input to real tty */
 	}
-
+flush:
 	flushlog();
     }
 }
@@ -909,12 +995,11 @@ static void socket_handler(int fd)
     case MAGIC_ASK_PWD:
 
 	if (!password) {
-	    password = (char*)shm_malloc(MAX_PASSLEN, MAP_LOCKED|MAP_ANONYMOUS|MAP_SHARED);
-	    if (!password)
+	    void *tmp = shm_malloc(MAX_PASSLEN+sizeof(int32_t));
+	    if (!tmp)
 		error("can not allocate string for password");
-	    pwsize = (int32_t*)mmap(NULL, sizeof(int32_t), PROT_READ|PROT_WRITE, MAP_LOCKED|MAP_ANONYMOUS|MAP_SHARED, -1, 0);
-	    if (pwsize == MAP_FAILED)
-		error("can not allocate integer for password length");
+	    password = (char *)tmp;
+	    pwsize = (int32_t *)(tmp+MAX_PASSLEN);
 	}
 	password[0] = '\0';
 	pwprompt = strdup(arg);
@@ -1130,19 +1215,41 @@ job:			/* Do not close connection for reply */
 /*
  * Do handle the connection in data
  */
+char* currenttty;
 static void ask_for_password(void)
 {
     struct timespec timeout = {0, 50000000};
     siginfo_t status = {};
     sigset_t set = {};
     struct console *c;
-    int wait = 0;
+    int wait;
+    size_t len;
 
-    if (!pwprompt)
+    if (!pwprompt && !*pwprompt)
 	return;
 
+    len = strlen(pwprompt);
+    len--;
+    if (pwprompt[len] == ' ') {
+	pwprompt[len] = '\0';
+	len--;
+    }
+    if (pwprompt[len] == ':') {
+	pwprompt[len] = '\0';
+	len--;
+    }
     set_signal(SIGCHLD, NULL, chld_handler);
 
+    wait = 200;
+    while (wait > 0 && (len = klogctl(SYSLOG_ACTION_SIZE_UNREAD, NULL, 0)) > 0) {
+	usleep(1000);
+        wait--;
+    }
+    wait = 0;
+
+    asking = 1;				/* Show only our question about password/passphrase */
+    klogctl(SYSLOG_ACTION_CONSOLE_OFF, NULL, 0);
+
     /* pwprompt */
     list_for_each_entry(c, &cons->node, node) {
 
@@ -1175,9 +1282,11 @@ static void ask_for_password(void)
 	    dup2(1, 2);
 	    dup2(c->fd, 0);
 	    dup2(c->fd, 1);
+	    currenttty = c->tty;
 
 	    list_for_each_entry(d, &cons->node, node)
 		if (d->fd >= 0) {
+		    (void)tcdrain(d->fd);
 		    close(d->fd);
 		    d->fd = -1;
 		}
@@ -1203,12 +1312,20 @@ static void ask_for_password(void)
 	    dup2(fdc, 1);
 	    close(fdc);
 
+	again:
 	    clear_input(0);
-
-	    if (c->flags & CON_SERIAL)
-		len = asprintf(&message, "\n\r%s: ", pwprompt);
+#if defined(__s390__) || defined(__s390x__)
+	    if ((major(c->dev) == 4 && minor(c->dev) >= 65) ||
+		(major(c->dev) == 227 && minor(c->dev) >= 1))
+		len = asprintf(&message, BOLD RED "\n\r%s: " NORM, pwprompt);
 	    else
-		len = asprintf(&message, BOLD "\r%s: " NORM, pwprompt);
+		len = asprintf(&message, "\n\r>> %s: ", pwprompt);
+#else
+	    if (c->flags & CON_SERIAL)
+		len = asprintf(&message, BOLD RED "\n\r%s: " NORM, pwprompt);
+	    else
+		len = asprintf(&message, BOLD RED "\r%s: " NORM, pwprompt);
+#endif
 	    if (len < 0) {
 		warn("can not set password prompt");
 		_exit(1);
@@ -1218,8 +1335,8 @@ static void ask_for_password(void)
 
 	    /* We read byte for byte */
 	    newtio = c->ctio;
-	    newtio.c_iflag &= ~(IUCLC|IXON|IXOFF|IXANY);
-	    newtio.c_lflag &= ~(ECHO|ECHOE|ECHOK|ECHONL|TOSTOP|ICANON|ISIG);
+	    newtio.c_lflag &= ~ECHO;
+	    newtio.c_lflag |= ECHONL;
 	    newtio.c_cc[VTIME] = 0;
 	    newtio.c_cc[VMIN] = 1;
 
@@ -1232,6 +1349,9 @@ static void ask_for_password(void)
 	    tcsetattr(0, TCSANOW, &c->ctio);
 	    safeout(1, "\n", 1, c->max_canon);
 
+            if (*pwsize == 0)
+                goto again;
+
 	    if (*pwsize < 0)
 		warn("can not read password");
 
@@ -1240,47 +1360,52 @@ static void ask_for_password(void)
 	}
     }
 
-    do {					/* Wait on any job if any */
+    do {				/* Wait on any job if any */
 	int ret;
 
 	status.si_pid = 0;
 	ret = waitid(P_ALL, 0, &status, WEXITED);
 
 	if (ret == 0)
-		break;
+	    break;
 
 	if (ret < 0) {
 	    if (errno == ECHILD)
 		break;
 	    if (errno == EINTR)
 		continue;
+	    error("can not wait on password asking process: %m");
+	    break;
 	}
 
-	error("can not wait on password asking process");
-
     } while (1);
 
+    asking = 0;				/* Now throw out any collected messages if any */
+    klogctl(SYSLOG_ACTION_CONSOLE_ON, NULL, 0);
+
     list_for_each_entry(c, &cons->node, node) {
 	if (c->fd < 0)
 	    continue;
 	if (c->pid < 0)
 	    continue;
-	if (c->pid == status.si_pid)		/* Remove first reply ... */
+	if (c->pid == status.si_pid)	/* Remove first reply ... */
 	    c->pid = -1;
 	else {
-	    kill(c->pid, SIGTERM);		/* and terminate the others */
+	    kill(c->pid, SIGTERM);	/* and terminate the others */
 	    wait++;
 	}
     }
 
     sigemptyset(&set);
-    sigaddset(&set, SIGCHLD);			/* On exit we'll see SIGCHLD */
+    sigaddset(&set, SIGCHLD);		/* On exit we'll see SIGCHLD */
 
     do {
 	int signum, ret;
 
-	if (!wait)
+	if (wait <= 0) {
+	    asking = 0;			/* Now throw out any collected messages if any */
 	    break;
+	}
 
 	status.si_pid = 0;
 	ret = waitid(P_ALL, 0, &status, WEXITED|WNOHANG);
