@@ -113,7 +113,7 @@ struct console {
 #define CON_BOOT	(8)
 #define CON_ANYTIME	(16) /* Safe to call when cpu is offline */
 #define CON_BRL		(32) /* Used for a braille device */
-#define CON_SERIAL	(64) /* serial line */
+#define CON_SERIAL	(64) /* Serial line */
 
 extern sigset_t omask;
 extern int final;
@@ -121,6 +121,7 @@ extern int epfd;
 extern int evmax;
 extern volatile sig_atomic_t signaled;
 extern volatile sig_atomic_t nsigsys;
+extern volatile sig_atomic_t asking;
 
 extern void remember_arg0(volatile char *arg0);
 extern ssize_t safein (int fd, void *ptr, size_t s);
@@ -138,6 +139,7 @@ extern void new_root(const char *root);
 
 /* devices.c */
 extern char *charname(const char *dev);
+extern char *chardev(const dev_t cons);
 
 /* dir.c */
 extern void pushd(const char * path);
@@ -178,7 +180,7 @@ extern void list_fd(const pid_t pid);
 extern ssize_t readpw(int fd, char *pass, int eightbit);
 
 /* shm.c */
-extern void* shm_malloc(size_t size, int flags);
+extern void* shm_malloc(size_t size);
 
 /* signals.c */
 extern void set_signal(int sig, struct sigaction *old, sighandler_t handler);
@@ -233,17 +235,18 @@ static inline void reset_virtual_console(struct termios *tp, int flags)
     /* Sane setting, allow eight bit characters, no carriage return delay
      * the same result as `stty sane cr0 pass8'
      */
-    tp->c_iflag |=  (BRKINT | ICRNL | IMAXBEL);
-    tp->c_iflag &= ~(IGNBRK | INLCR | IGNCR | IXOFF | IUCLC | IXANY | ISTRIP);
+    tp->c_iflag |=  (ICRNL | IXON);
+    tp->c_iflag &= ~(IGNBRK | BRKINT | IGNPAR | PARMRK | INPCK | ISTRIP| \
+		     INLCR | IGNCR | IXOFF | IUCLC | IXANY | IMAXBEL);
     tp->c_oflag |=  (OPOST | ONLCR | NL0 | CR0 | TAB0 | BS0 | VT0 | FF0);
     tp->c_oflag &= ~(OLCUC | OCRNL | ONOCR | ONLRET | OFILL | \
-    		    NLDLY|CRDLY|TABDLY|BSDLY|VTDLY|FFDLY);
-    tp->c_lflag |=  (ISIG | ICANON | IEXTEN | ECHO|ECHOE|ECHOK|ECHOKE|ECHOCTL);
-    tp->c_lflag &= ~(ECHONL|ECHOPRT | NOFLSH | TOSTOP);
+    		     NLDLY|CRDLY|TABDLY|BSDLY|VTDLY|FFDLY);
+    tp->c_lflag |=  (ISIG | ICANON | IEXTEN | ECHO|ECHOE|ECHOK|ECHOKE);
+    tp->c_lflag &= ~(ECHONL|ECHOPRT | NOFLSH | TOSTOP | FLUSHO | EXTPROC | ECHOCTL);
 
     if ((flags & UL_TTY_KEEPCFLAGS) == 0) {
 	tp->c_cflag |=  (CREAD | CS8 | HUPCL);
-	tp->c_cflag &= ~(PARODD | PARENB);
+	tp->c_cflag &= ~(PARODD | PARENB | CMSPAR | CLOCAL | CRTSCTS);
     }
 #ifdef OFDEL
     tp->c_oflag &= ~OFDEL;
